export interface PairHeapNode<T> {
    /** Value */
    v: T;
    /** Siblings */
    s: PairHeapNode<T> | undefined;
    /** Children */
    c: PairHeapNode<T> | undefined;
}
/**
 * Compare Functions
 * Compares two values a and b.
 * Meaning of return value:
 * `v <= 0`: `a` is ahead of `b`
 * `v > 0`: `b` is ahead of `a`
 * @param a - item a
 * @param b - item b
 * @returns a number
 */
export type CompareFn<T> = (a: T, b: T) => number;
export declare class PairingHeap<T> implements IterableIterator<T> {
    readonly compare: CompareFn<T>;
    private _heap;
    private _size;
    constructor(compare: CompareFn<T>);
    /** Add an item to the heap. */
    add(v: T): this;
    /** take an item from the heap. */
    dequeue(): T | undefined;
    /** Add items to the heap */
    concat(i: Iterable<T>): this;
    /** get the next value */
    next(): IteratorResult<T>;
    /** peek at the next value without removing it. */
    peek(): T | undefined;
    [Symbol.iterator](): IterableIterator<T>;
    /** alias of `size` */
    get length(): number;
    /** number of entries in the heap. */
    get size(): number;
}
declare function insert<T>(compare: CompareFn<T>, heap: PairHeapNode<T> | undefined, v: T): PairHeapNode<T>;
declare function merge<T>(compare: CompareFn<T>, a: PairHeapNode<T>, b: PairHeapNode<T>): PairHeapNode<T>;
declare function mergeSiblings<T>(compare: CompareFn<T>, n: PairHeapNode<T>): PairHeapNode<T>;
export declare const heapMethods: {
    insert: typeof insert;
    merge: typeof merge;
    mergeSiblings: typeof mergeSiblings;
};
export {};
//# sourceMappingURL=PairingHeap.d.ts.map